/*
 * timeConstant.c:
 *	timeConstant program trying to measure the time constant of an 
 *      RC circuit
 *      A 10 k Ohms resitor is connected to 3.3V on one end
 *      and to a 2.2 uF capacitor on the other
 *      The other end of the capacitor is connected to ground
 *      The connection point between resistor and capacitor is connected
 *	to the first GPIO pin.

 *      Connect the - pin of the photo resistor via a 330 Ohms resistor to 3.3V
 *      Connect pin S of the photo resistor to the + pole of the 2.2 uF capacitor
 *      Connect oin S of the photo resistor to gpio 0 (marked 17 on the cobbler) 
 *      Connect the - pole of the capacitor to ground 
 *
 * Copyright (c)Uli Raich (uli.raich@cern.ch
 * This program is part of a workshop on embedded systems held during the
 * winter semester 2017 at the University of Cape Coast, Ghana
 */

#include <stdio.h>
#include <wiringPi.h>
#include <time.h>

// Resistor and capacitor connection point - wiringPi pin 0 is BCM_GPIO 17.

#define	RC_POINT	0         /* connection point */

int main (void)
{
  struct timespec timeSpec, *timePtr;
  struct timespec startTime, stopTime;
  
  int timeOut;
  timePtr = &timeSpec;
  long resolution;

  printf ("Raspberry Pi measure RC time constant\n") ;

  clock_getres(CLOCK_REALTIME,timePtr);
  printf ("Resolution is %d ns\n",timePtr->tv_nsec);

  wiringPiSetup () ;
  pinMode (RC_POINT, OUTPUT) ;
  
  /* discharge the capacitor */
  
  digitalWrite(RC_POINT,0);
  delay(100);                      /* wait for 100 ms */

  clock_gettime(CLOCK_REALTIME,&startTime);
  timeOut = 100000000;
  pinMode(RC_POINT,INPUT);         /* switch the pin to input */
  
  /* measure the time it takes to charge the capacitor */
  
  while (!digitalRead(RC_POINT)) {
    if (! --timeOut)
       break;
  }
  
  clock_gettime(CLOCK_REALTIME,&stopTime);

  if (!timeOut)
    printf("timeOut!\n");
  else {
    printf("Start time: %d, Stop time %d\n",stopTime.tv_nsec,startTime.tv_nsec);
    printf("Start time secs, %d, Stop time secs: %d\n",
	   stopTime.tv_sec, startTime.tv_sec);
    if ((stopTime.tv_nsec - startTime.tv_nsec) < 0)
      printf("negative: %10.4f\n",
	   (double) (stopTime.tv_nsec - startTime.tv_nsec)/1000000);
    printf("Time to charge capacitor: %10.4f ms\n",
	   (double) (stopTime.tv_sec  - startTime.tv_sec)*1000 +
	   (double) (stopTime.tv_nsec - startTime.tv_nsec)/1000000);
  }  
}
